try:
    # python3
    from base64 import a85encode, a85decode
except ImportError:
    # python2
    from mom.codec.base85 import b85decode, b85encode as a85encode

    def a85decode(*args, **kwargs):
        try:
            return b85decode(*args, **kwargs)
        except (TypeError, OverflowError) as e:
            # behave like py3
            raise ValueError(e)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of the Kount python sdk project
# https://github.com/Kount/kount-ris-python-sdk/)
# Copyright (C) 2017 Kount Inc. All Rights Reserved.
"Address class - representing a street address"

from __future__ import absolute_import, unicode_literals, division, \
    print_function

from kount.version import VERSION
from kount.config import SDKConfig

__author__ = SDKConfig.SDK_AUTHOR
__version__ = VERSION
__maintainer__ = SDKConfig.SDK_MAINTAINER
__email__ = SDKConfig.MAINTAINER_EMAIL
__status__ = SDKConfig.STATUS


class Address(object):
    """A class representing a street address.
       Keyword arguments:
            address1 - Address 1 (default empty string)
            address2 - Address 2 (default empty string)
            city - City (default empty string)
            state - State (default empty string)
            postal_code - Postal code (default empty string)
            country - Country (default empty string)
            premise - Premise (default empty string)
            street - Street (default empty string)
    """

    def __init__(self, address1="", address2="", city="", state="",
                 postal_code="", country="", premise="", street=""):
        """Address constructor."""
        self.address1 = address1
        self.address2 = address2
        self.city = city
        self.state = state
        self.postal_code = postal_code
        self.country = country
        self.premise = premise
        self.street = street



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of the Kount python sdk project
# https://github.com/Kount/kount-ris-python-sdk/)
# Copyright (C) 2017 Kount Inc. All Rights Reserved.
"""RIS Request superclass for Inquiry and Update"""

import logging

from .config import SDKConfig
from .util.khash import Khash
from .util import payment as payments
from .version import VERSION

__author__ = SDKConfig.SDK_AUTHOR
__version__ = VERSION
__maintainer__ = SDKConfig.SDK_MAINTAINER
__email__ = SDKConfig.MAINTAINER_EMAIL
__status__ = SDKConfig.STATUS

LOG = logging.getLogger('kount.request')


class _RisType(object):

    __CACHED_ATTRS = None

    @classmethod
    def is_valid(cls, val):
        attrs = cls.__CACHED_ATTRS
        if attrs is None:
            attrs = {
                v for k, v in vars(cls).items() if not k.startswith('_')
            }
            cls.__CACHED_ATTRS = attrs
        return val in attrs


class AuthStatus(_RisType):
    """Authorization status"""
    APPROVE = 'A'
    DECLINE = 'D'
    REVIEW = 'R'
    ESCALATE = "E"
    REVIEW_TIMEOUT = 'X'
    APPROVED_DECLINED_TIMEOUT = 'Y'
    ELEVATED = 'C'


class BankcardReply(_RisType):
    """Bankcard Reply"""
    MATCH = 'M'
    NO_MATCH = 'N'
    UNAVAILABLE = 'X'


class Gender(_RisType):
    """gender"""
    MALE = 'M'
    FEMALE = 'F'


class AddressType(_RisType):
    """address type"""
    BILLING = 'B'
    SHIPPING = 'S'


class ShippingType(_RisType):
    """
    "SD". Same day shipping type.
    "ND". Next day shipping type.
    "2D". Second day shipping type.
    "ST". Standard shipping type.
    """
    SAME_DAY = 'SD'
    NEXT_DAY = 'ND'
    SECOND_DAY = '2D'
    STANDARD = "ST"


class RefundChargebackStatus(_RisType):
    """Refund charge back status.
    R - The transaction was a refund.
    C - The transaction was a chargeback.
    """
    REFUND = 'R'
    CHARGEBACK = 'C'


class MerchantAcknowledgment(_RisType):
    """merchant acknowledgment
    "Y". The product expects to ship.
    "N". The product does not expect to ship.
    """
    FALSE = 'N'
    TRUE = 'Y'


class CurrencyType(_RisType):
    """Currency type object
    "USD". United States Dollars
    "EUR". European currency unit
    "CAD". Canadian Dollar
    "AUD". Australian Dollar
    "JPY". Japanese Yen
    "HKD". Honk Kong Dollar
    "NZD". New Zealand Dollar
    """
    USD = 'USD'
    EUR = 'EUR'
    CAD = 'CAD'
    AUD = 'AUD'
    JPY = 'JPY'
    HKD = 'HKD'
    NZD = 'NZD'


class InquiryMode(_RisType):
    """
    "Q". Default inquiry mode, internet order type.
    "P". Phone order type.
    "W". Kount Central Mode W - Full Inquiry [W]ith thresholds.
    "J". Kount Central Mode J - Fast Inquiry [J]ust thresholds.
    """
    DEFAULT = 'Q'
    PHONE = 'P'
    WITH_THRESHOLDS = 'W'
    JUST_THRESHOLDS = 'J'


class Request(object):
    """RIS Request superclass for Inquiry and Update."""

    def __init__(self):
        """Map containing data that will be sent to RIS."""
        self.params = dict()
        self.payment = None
        self.close_on_finish = None

    def set_param(self, key, value):
        """Set a parm for the request.
        Args:
           key - The key for the parm
           value - The value for the parm
        """
        self.params[key] = value
        LOG.debug("%s = %s", key, value)

    def set_khash_payment_encoding(self, enabled=True):
        """Set KHASH payment encoding.
        Arg: enabled Boolean
        """
        self.set_param("PENC", "KHASH" if enabled else "")

    def set_version(self, version):
        """Set the version number.
        Args: version - The SDK version
        """
        self.set_param("VERS", version)

    def set_session_id(self, session_id):
        """Set the session id. Must be unique over a 30-day span
        Args: session_id -  Id of the current session
        """
        self.set_param("SESS", session_id)

    def set_merchant(self, merchant_id):
        """Set the merchant id.
        Args: merchant_id - Merchant ID
        """
        self.set_param("MERC", merchant_id)

    def set_kount_central_customer_id(self, customer_id):
        """Set the Kount Central Customer ID.
        Args: customer_id - KC Customer ID
        """
        self.set_param("CUSTOMER_ID", customer_id)

    def set_order_number(self, order_number):
        """Set the order number.
        Args: order_number - Merchant unique order number
        """
        self.set_param("ORDR", order_number)

    def set_merchant_acknowledgment(self, ma_type):
        """Set the merchant acknowledgment.
        Merchants acknowledgement to ship/process the order.
        The MACK field must be set as MerchantAcknowledgment.TRUE
        if personal data is to be
        collected to strengthen the score.
        Args: ma_type - merchant acknowledgment type
        """
        if MerchantAcknowledgment.is_valid(ma_type):
            self.set_param("MACK", ma_type)
        else:
            raise ValueError("Invalid MerchantAcknowledgment = %s" % ma_type)

    def set_authorization_status(self, auth_status):
        """Set the Authorization Status.
        Authorization Status returned to merchant from processor.
        Acceptable values for the
        AUTH field are AuthStatus. In orders where AUTH=A will
        aggregate towards order velocity of the persona while
        orders where AUTH=D will
        decrement the velocity of the persona.
        Args: auth_status - Auth status by issuer
        """
        if AuthStatus.is_valid(auth_status):
            self.set_param("AUTH", auth_status)
        else:
            raise ValueError("Invalid AuthStatus value %s" % auth_status)

    def set_avs_zip_reply(self, avs_zip_reply):
        """Set the Bankcard AVS zip code reply.
        Address Verification System Zip Code verification response
        returned to merchant from
        processor. Acceptable values are BCRSTAT.
        Args: avs_zip_reply - Bankcard AVS zip code reply
        """
        if BankcardReply.is_valid(avs_zip_reply):
            self.set_param("AVSZ", avs_zip_reply)
        else:
            raise ValueError('Invalid BankcardReply = %s' % avs_zip_reply)

    def set_avs_address_reply(self, avs_address_reply):
        """Set the Bankcard AVS street addres reply.
        Address Verification System Street verification response
        returned to merchant from processor. Acceptable values are BCRSTAT.
        Args: avs_address_reply - Bankcard AVS street address reply
        """
        if BankcardReply.is_valid(avs_address_reply):
            self.set_param("AVST", avs_address_reply)
        else:
            raise ValueError('Invalid BankcardReply = %s' % avs_address_reply)

    def set_avs_cvv_reply(self, cvv_reply):
        """Set the Bankcard CVV/CVC/CVV2 reply.
        Card Verification Value response returned to merchant from processor.
        Acceptable values are BCRSTAT
        Args: cvv_reply -  Bankcard CVV/CVC/CVV2 reply
        """
        if BankcardReply.is_valid(cvv_reply):
            self.set_param("CVVR", cvv_reply)
        else:
            raise ValueError('Invalid BankcardReply = %s' % cvv_reply)

    def set_payment(self, payment):
        """ Set a payment.
            Depending on the payment type, various request parameters are set:
            PTOK, PTYP, LAST4.
            If payment token hashing is not possible, the PENC parameter is set
            to empty string.
            Args: payment -  Payment
        """
        khasher = Khash.get()
        if "PENC" in self.params \
                and not isinstance(payment, payments.NoPayment) \
                and not payment.khashed:
            try:
                if not self.params.get("MERC"):
                    raise ValueError("merchant_id not set")
                if isinstance(payment, payments.GiftCardPayment):
                    merchant_id = int(self.params["MERC"])
                    payment.payment_token = khasher.hash_gift_card(
                        merchant_id, payment.payment_token)
                else:
                    payment.payment_token = khasher.hash_payment_token(
                        payment.payment_token)
                payment.khashed = True
                self.set_param("PENC", "MASK")
                LOG.debug("payment.khashed=%s", payment.khashed)
            except ValueError as nfe:
                LOG.debug("Error converting Merchant ID to integer"
                          " value. Set a valid Merchant ID. %s",
                          str(nfe))
                raise nfe
            except Exception as nsae:
                LOG.debug("Unable to create payment token hash. Caught %s "
                          "KHASH payment encoding disabled", str(nsae))
                # Default to plain text payment tokens
                self.params["PENC"] = ""
        if khasher.khashed(payment.payment_token):
            self.set_param("PENC", "KHASH")
        self.set_param("PTOK", payment.payment_token)
        self.set_param("PTYP", payment.payment_type)
        self.set_param("LAST4", payment.last4)

    @staticmethod
    def _mask_token(token):
        """Encodes the provided payment token according to the MASK
           encoding scheme
           Args: token -  the Payment token for this request
           return - MASK-encoded token
        """
        encoded = token[0:6]
        for _ in range(6, len(token) - 4, 1):
            encoded.append('X')
        encoded.append(token[-4:])
        LOG.debug("mask_token = %s", token)
        return encoded

    def set_payment_by_type(self, ptyp, ptok):
        """ Set a payment by payment type and payment token.
        The payment type parameter provided is checked
        if it's one of the predefined payment types
        and Payment is created appropriately
        Args: ptyp - See SDK documentation for a list of accepted payment types
        ptok - The payment token
        """
        cls = {
            "BLML": payments.BillMeLaterPayment,
            'CARD': payments.CardPayment,
            'CHECK': payments.CheckPayment,
            'GIFT': payments.GiftCardPayment,
            'GOOG': payments.GooglePayment,
            'GDMP': payments.GreenDotMoneyPakPayment,
            'NONE': payments.NoPayment,
            'PYPL': payments.PaypalPayment,
        }.get(ptyp)
        if cls is None:
            cls = payments.Payment
        self.set_payment(cls(ptyp, ptok))

    def set_masked_payment(self, payment):
        """ Sets a card payment and masks the card number in the following way:
        First 6 characters remain as they are, following characters up to the
        last 4 are replaced with the 'X' character, last 4 characters
        remain as they are.
        If the provided Payment parameter is not a card payment,
        standard encoding will be applied.
        This method sets the following RIS Request fields:
        PTOK, PTYP, LAST4, PENC.
        Args: payment - card payment
        """
        token = payment.payment_token
        if isinstance(self.payment, payment.CardPayment) and \
                not payment.khashed:
            token = self._mask_token(token)
            self.set_param("PTOK", token)
            self.set_param("PTYP", payment.payment_type)
            self.set_param("LAST4", payment.last4)
            self.set_param("PENC", "MASK")
        else:
            self.set_param("PTOK", token)
            LOG.debug("Payment Masked: provided payment is not "
                      "a CardPayment, applying khash instead of masking")
            self.set_payment_by_type(payment, token)

    def is_set_khash_payment_encoding(self):
        """Check if KHASH payment encoding has been set.
           return boolean TRUE when set.
        """
        encoded = self.params.get("PENC") == "KHASH"
        LOG.debug("is_set_khash_payment_encoding = %s", encoded)
        return encoded

    def set_close_on_finish(self, close_on_finish):
        """Set a flag for the request transport.
           Arg: close_on_finish - Sets the close_on_finish flag
           return boolean TRUE when set.
        """
        self.close_on_finish = close_on_finish
        LOG.debug("close_on_finish = %s", close_on_finish)


class UpdateMode(_RisType):
    """UpdateMode - U, X"""
    NO_RESPONSE = 'U'
    WITH_RESPONSE = 'X'


class Update(Request):
    """RIS update class.
     defaults to update_mode WithResponse.
     """

    def __init__(self):
        super(Update, self).__init__()
        self.set_mode(UpdateMode.NO_RESPONSE)
        self.params['VERS'] = SDKConfig.SDK_VERSION
        # self.params["SDK"] = "python"
        self.set_khash_payment_encoding(True)

    def set_mode(self, mode):
        """Set the mode.
        Args - mode - Mode of the request
        """
        if UpdateMode.is_valid(mode):
            self.params["MODE"] = mode
        else:
            raise ValueError("Invalid UpdateMode: %s" % mode)

    def set_transaction_id(self, transaction_id):
        """Set the transaction id.
        Arg - transaction_id, String Transaction id
        """
        self.params["TRAN"] = transaction_id

    def set_refund_chargeback_status(self, rc_status):
        """Set the Refund/Chargeback status: R = Refund C = Chargeback.
        Arg - rc_status, String Refund or chargeback status
        """
        if RefundChargebackStatus.is_valid(rc_status):
            self.params["RFCB"] = rc_status
        else:
            raise ValueError("Invalid RefundChargebackStatus: %s" % rc_status)




#!/usr/bin/env python
"class Payment - RIS payment type object"
# -*- coding: utf-8 -*-
# This file is part of the Kount python sdk project
# https://github.com/Kount/kount-ris-python-sdk/)
# Copyright (C) 2017 Kount Inc. All Rights Reserved.

from kount.util.khash import Khash
from kount.version import VERSION
from kount.config import SDKConfig

__author__ = SDKConfig.SDK_AUTHOR
__version__ = VERSION
__maintainer__ = SDKConfig.SDK_MAINTAINER
__email__ = SDKConfig.MAINTAINER_EMAIL
__status__ = SDKConfig.STATUS


class Payment(object):
    """RIS payment type object.
    Args:
    payment_type - Payment type.
    payment_token - Payment token
    khashed - Indicates whether payment token is khashed.
    True if payment token is khashed.
    """
    def __init__(self, payment_type=None, payment_token=None, khashed=True):
        """Constructor for a payment that accepts the payment ID.
        Calculate and set the payment token LAST4 value.
        last4 - Last 4 characters of payment token"""
        self.last4 = "NONE"
        self.payment_token = None
        if payment_type is not None:
            self._payment_type = str(payment_type)
        else:
            self._payment_type = "NONE"

        if payment_token is not None:
            self.payment_token = str(payment_token)
            if len(self.payment_token) >= 4:
                self.last4 = self.payment_token[-4:]
            if khashed:
                self.khashed = self.khash_token()
        self.khashed = khashed and Khash.khashed(self.payment_token)

    @property
    def payment_type(self):
        return self._payment_type

    def khash_token(self):
        "hash the payment_token, return True if khashed, else raise ValueError"
        k = Khash.get()
        self.payment_token = k.hash_payment_token(
            token=self.payment_token)
        if k.khashed(self.payment_token):
            return True
        raise ValueError("payment_token [%s] is not khashed" %
                         self.payment_token)


class GiftCardPayment(Payment):
    """Sets the PTYP parameter to GIFT,
    params: gift_card_number,
     khashed - boolean"""
    def __init__(self, gift_card_number, khashed=True):
        super(GiftCardPayment, self).__init__(
            payment_type="GIFT",
            payment_token=gift_card_number,
            khashed=khashed)


class GooglePayment(Payment):
    """Sets the PTYP parameter to "GIFT".
    params: google_payment_id - Google payment ID
                khashed - boolean"""
    def __init__(self, google_payment_id, khashed=True):
        super(GooglePayment, self).__init__(
            payment_type="GOOG",
            payment_token=google_payment_id,
            khashed=khashed)


class GreenDotMoneyPakPayment(Payment):
    """Sets the PTYP parameter to "GDMP".
    params: green_dot_mp_payment - Green Dot MoneyPak payment ID number
            khashed - boolean"""
    def __init__(self, green_dot_mp_payment, khashed=True):
        super(GreenDotMoneyPakPayment, self).__init__(
            payment_type="GDMP",
            payment_token=green_dot_mp_payment,
            khashed=khashed)


class NoPayment(Payment):
    """No payment type. Sets the PTYP parameter to "NONE", not khashed"""
    def __init__(self, *args, **kwargs):
        super(NoPayment, self).__init__(
            payment_type=None,
            payment_token=None,
            khashed=False)


class CheckPayment(Payment):
    """Sets the PTYP parameter to "CHEK".
    params: micr - The MICR (Magnetic Ink Character Recognition)
                line on the check.
            khashed - boolean
    """
    def __init__(self, micr, khashed=True):
        super(CheckPayment, self).__init__(
            payment_type="CHEK",
            payment_token=micr,
            khashed=khashed)


class PaypalPayment(Payment):
    """paypal payment - accepts the paypal payment ID.
    Sets the PTYP parameter to "PYPL".
    params: paypal_payment_id - Paypal payment ID
            khashed - boolean
    """
    def __init__(self, paypal_payment_id, khashed=True):
        super(PaypalPayment, self).__init__(
            payment_type="PYPL",
            payment_token=paypal_payment_id,
            khashed=khashed)


class CardPayment(Payment):
    """credit card payment
    Sets the PTYP parameter to "CARD".
    params: card_number - The card number
            khashed - boolean
    """

    def __init__(self, card_number, khashed=True):
        super(CardPayment, self).__init__(
            payment_type="CARD",
            payment_token=card_number,
            khashed=khashed)


class BillMeLaterPayment(Payment):
    """bill me later payment
    Sets the PTYP parameter to "BLML".
    params: payment_id - The payment ID,
            khashed - boolean
    """
    def __init__(self, payment_id, khashed=True):
        super(BillMeLaterPayment, self).__init__(
            payment_type="BLML",
            payment_token=payment_id,
            khashed=khashed)


class ApplePay(Payment):
    """Apple Pay
    Sets the PTYP parameter to "APAY".
    params: payment_id - apay,
            khashed - boolean
    """
    def __init__(self, payment_id, khashed=True):
        super(ApplePay, self).__init__(
            payment_type="APAY",
            payment_token=payment_id,
            khashed=khashed)


class BPayPayment(Payment):
    """BPay Payment
    Sets the PTYP parameter to "BPAY".
    params: payment_id - bpay,
            khashed - boolean
    """
    def __init__(self, payment_id, khashed=True):
        super(BPayPayment, self).__init__(
            payment_type="BPAY",
            payment_token=payment_id,
            khashed=khashed)


class CarteBleuePayment(Payment):
    """Carte Bleue Payment
    Sets the PTYP parameter to "CARTE_BLEUE".
    params: payment_id - Carte Bleue id,
            khashed - boolean
    """
    def __init__(self, payment_id, khashed=True):
        super(CarteBleuePayment, self).__init__(
            payment_type="CARTE_BLEUE",
            payment_token=payment_id,
            khashed=khashed)


class ELVPayment(Payment):
    """ELV Payment
    Sets the PTYP parameter to "ELV".
    params: payment_id - ELV id,
            khashed - boolean
    """
    def __init__(self, payment_id, khashed=True):
        super(ELVPayment, self).__init__(
            payment_type="ELV",
            payment_token=payment_id,
            khashed=khashed)


class GiroPayPayment(Payment):
    """GIROPAY Payment
    Sets the PTYP parameter to "GIROPAY".
    params: payment_id - id,
            khashed - boolean
    """
    def __init__(self, payment_id, khashed=True):
        super(GiroPayPayment, self).__init__(
            payment_type="GIROPAY",
            payment_token=payment_id,
            khashed=khashed)


class InteracPayment(Payment):
    """Interac Payment
    Sets the PTYP parameter to "INTERAC".
    params: payment_id - id,
            khashed - boolean
    """
    def __init__(self, payment_id, khashed=True):
        super(InteracPayment, self).__init__(
            payment_type="INTERAC",
            payment_token=payment_id,
            khashed=khashed)


class MercadoPagoPayment(Payment):
    """Mercado Pago Payment
    Sets the PTYP parameter to "MERCADE_PAGO".
    params: payment_id - id,
            khashed - boolean
    """
    def __init__(self, payment_id, khashed=True):
        super(MercadoPagoPayment, self).__init__(
            payment_type="MERCADE_PAGO",
            payment_token=payment_id,
            khashed=khashed)


class NetellerPayment(Payment):
    """Neteller Payment
    Sets the PTYP parameter to "NETELLER".
    params: payment_id - id,
            khashed - boolean
    """
    def __init__(self, payment_id, khashed=True):
        super(NetellerPayment, self).__init__(
            payment_type="NETELLER",
            payment_token=payment_id,
            khashed=khashed)


class PoliPayment(Payment):
    """POLi Payment
    Sets the PTYP parameter to "POLI".
    params: payment_id - id,
            khashed - boolean
    """
    def __init__(self, payment_id, khashed=True):
        super(PoliPayment, self).__init__(
            payment_type="POLI",
            payment_token=payment_id,
            khashed=khashed)


class SEPAPayment(Payment):
    """Single Euro Payments Area Payment
    Sets the PTYP parameter to "SEPA".
    params: payment_id - id,
            khashed - boolean
    """
    def __init__(self, payment_id, khashed=True):
        super(SEPAPayment, self).__init__(
            payment_type="SEPA",
            payment_token=payment_id,
            khashed=khashed)


class SofortPayment(Payment):
    """Sofort Payment
    Sets the PTYP parameter to "SOFORT".
    params: payment_id - id,
            khashed - boolean
    """
    def __init__(self, payment_id, khashed=True):
        super(SofortPayment, self).__init__(
            payment_type="SOFORT",
            payment_token=payment_id,
            khashed=khashed)


class TokenPayment(Payment):
    """Token Payment
    Sets the PTYP parameter to "TOKEN".
    params: payment_id - id,
            khashed - boolean
    """
    def __init__(self, payment_id, khashed=True):
        super(TokenPayment, self).__init__(
            payment_type="TOKEN",
            payment_token=payment_id,
            khashed=khashed)


class SkrillPayment(Payment):
    """Skrill/Moneybookers Payment
    Sets the PTYP parameter to "SKRILL".
    params: payment_id - id,
            khashed - boolean
    """
    def __init__(self, payment_id, khashed=True):
        super(SkrillPayment, self).__init__(
            payment_type="SKRILL",
            payment_token=str(payment_id),
            khashed=khashed)


def NewPayment(payment_type, payment_token, khashed=True):
    """User-defined payment type
    Sets the PTYP parameter to desired parameter.
    params: payment_type
            payment_token
            khashed - boolean
    """
    return Payment(payment_type, payment_token, khashed=khashed)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of the Kount python sdk project
# https://github.com/Kount/kount-ris-python-sdk/)
# Copyright (C) 2017 Kount Inc. All Rights Reserved.
import logging
from collections import namedtuple

from .version import VERSION
from .config import SDKConfig

__author__ = SDKConfig.SDK_AUTHOR
__version__ = VERSION
__maintainer__ = SDKConfig.SDK_MAINTAINER
__email__ = SDKConfig.MAINTAINER_EMAIL
__status__ = SDKConfig.STATUS

LOG = logging.getLogger('kount.response')

KcEvent = namedtuple('KcEvent', 'decision expression code')


class Response(object):
    """RIS response data class."""
    def __init__(self, params):
        """Constructor for a response object that accepts a map of response data
        Arg - params - Response parameters:

        "VERS" -  Get the version number.
        "MODE"
        "MERC" - merchant id
        "SESS" - session id.
        "TRAN" - Get the transaction id.
        "ORDR" -Get the merchant order number.
        "ERRO" - Get a possible error code.
        "AUTO" - Get the RIS auto response (A/R/D).
        "REASON_CODE" - Get the RIS reason for the response.
        "SCOR" - Get the Kount score.
        "OMNISCORE" - Get the Kount Omniscore.
        "GEOX" - Get the geox.
        "BRND" - Get the credit card brand.
        "VELO" - Get the 6 hour velocity.
        "VMAX" - Get the 6 week velocity.
        "NETW" - Get the network type.
        "KYCF" - Get the know your customer flag.
        "REGN" - Get the region the remote device is located in.
        "KAPT" - Get the Kaptcha flag: enabled upon request and for
                 when RIS has no
        "SITE" - Get the site ID.
        "PROXY" - Get a string representing whether the remote device is
            using a proxy. return "Y" or "N"
        "EMAILS" - Get the number of transactions associated with the email.
        "HTTP_COUNTRY" - Get the two character country code setting in
                         the remote device's
        "TIMEZONE"- Get a string representing the time zone of the customer
                    as a 3 digit
        "CARDS" - Get the number of transactions associated with the credit card
        "PC_REMOTE" - Get a string representing whether the end device is a
            remotely controlled, return "Y" or "N"
        "DEVICES" - Get the number of transactions associated with the
            particular device.
        "DEVICE_LAYERS" - Get a string representing the five layers
            (OS, SSL, HTTP, Flash, JavaScript) of the remote device.
        "MOBILE_FORWARDER" - Get the mobile device's wireless appl. protocol.
        "VOICE_DEVICE" - Get a string representing whether or not the remote
            device is voice controlled. return "Y" or "N"
        "LOCALTIME" - Get local time of the remote device in the YYYY-MM-DD.
        "MOBILE_TYPE" - Get the mobile device type.
        "FINGERPRINT" - Get the device finger print.
        "FLASH" - Get a string representing whether or not the remote device
            allows flash. return "Y" or "N"
        "LANGUAGE" - Get the language setting on the remote device.
        "COUNTRY" - Get the remote device's country of origin as a 2 characters
        "JAVASCRIPT" - Get a string representing whether the remote device
            allows JavaScript. return "Y" or "N"
        "COOKIES" - Get a string representing whether the remote device allows
            cookies. return "Y" or "N"
        "MOBILE_DEVICE" - Get a string representing whether the remote device
            is a mobile device. return "Y" or "N"
        "PIP_IPAD" - Get pierced IP address.
        "PIP_LAT" - Get latitude of pierced IP address.
        "PIP_LON" - Get longitude of pierced IP address.
        "PIP_COUNTRY" - Get country of pierced IP address.
        "PIP_REGION" - Get region of pierced IP address.
        "PIP_CITY" - Get city of pierced IP address.
        "PIP_ORG" - Get organization of pierced IP address.
        "IP_IPAD" - Get proxy IP address.
        "IP_LAT" - Get latitude of proxy IP address.
        "IP_LON" - Get longitude of proxy IP address.
        "IP_COUNTRY" - Get country of proxy IP address.
        "IP_REGION" - Get region of proxy IP address.
        "IP_CITY" - Get city of proxy IP address.
        "IP_ORG" - Get organization of proxy IP address.
        "DDFS" - Get date device first seen.
        "UAS" - Get user agent string.
        "DSR" - Get device screen resolution.
        "OS" - Get operating system (derived from user agent string).
        "BROWSER" - Get browser (derived from user agent string).
        "KC_CUSTOMER_ID" - Get the Kount Central Customer ID.
        "KC_DECISION" - Get the Kount Central Decision.
        "KC_WARNING_COUNT" - Get the number of KC warnings associated with
                             the response.
        "KC_ERROR_COUNT" - Get the number of KC errors associated
                            with the response.
        "KC_TRIGGERED_COUNT" - Get the number of KC events associated
                                with the response.
        "COUNTERS_TRIGGERED" - Get the number of rules counters triggered
                                in the response.
        "MASTERCARD" - Get MasterCard Fraud Score associated
                        with the RIS transaction.
            Please contact your Kount representative to enable support for this
            feature in your merchant account.
        "RULES_TRIGGERED" - A RIS response will always contain the field
                            RULES_TRIGGERED which will be set to zero
                            if there are no rules triggered.
        "WARNING_COUNT" - Get the number of warnings associated
                            with the response.
        "ERROR_COUNT" - Get the number of errors associated with the response.
        """
        self.params = params
        LOG.debug("RIS response init = %s", params)

    def get_kc_warnings(self):
        """Get an List of the KC warnings returned by this Response."""
        warnings = []
        for i in range(int(self.params.get("KC_WARNING_COUNT", 0))):
            warn = self.params.get("KC_WARNING_%s" % i)
            if warn:
                warnings.append(warn)
        LOG.debug("RIS get_kc_warnings = %s", warnings)
        return warnings

    def get_kc_errors(self):
        """get kc errors"""
        errors = []
        for i in range(int(self.params.get("KC_ERROR_COUNT", 0))):
            err = self.params.get("KC_ERROR_%s" % i)
            if err:
                errors.append(err)
        LOG.debug("RIS get_kc_errors = %s", errors)
        return errors

    def get_kc_events(self):
        """Get an ArrayList of the KC events returned by this Response."""
        events = []
        for i in range(int(self.params.get("KC_TRIGGERED_COUNT", 0))):
            idx = i + 1
            decision = self.params.get("KC_EVENT_%s_DECISION" % idx)
            expression = self.params.get("KC_EVENT_%s_EXPRESSION" % idx)
            code = self.params.get("KC_EVENT_%s_CODE" % idx)
            events.append(KcEvent(decision, expression, code))
        LOG.debug("RIS get_kc_events = %s", events)
        return events

    def get_kc_decision(self):
        return self.params.get('KC_DECISION')

    def get_rules_triggered(self):
        """Get a Map of the rules triggered by this Response."""
        rules = {}
        rules_triggered_count = int(self.params.get("RULES_TRIGGERED", 0))
        for i in range(rules_triggered_count):
            rule_id = self.params["RULE_ID_%s" % i]
            rules[rule_id] = self.params["RULE_DESCRIPTION_%s" % i]
        LOG.debug("RIS get_rules_triggered = %s", rules)
        return rules

    def get_session_id(self):
        """ Get the session id. """
        return self.params.get('SESS')

    def get_transaction_id(self):
        """ Get the transaction id. """
        return self.params.get('TRAN')

    def get_order_id(self):
        """ Get the merchant order number. """
        return self.params.get('ORDR')

    def get_mode(self):
        """ Get the mode. """
        return self.params.get('MODE')

    def get_auto(self):
        """ Get the RIS auto response (A/R/D). """
        return self.params.get('AUTO')

    def get_velo(self):
        """ Get the 6 hour velocity. """
        return self.params.get('VELO')

    def get_vmax(self):
        """ Get the 6 week velocity. """
        return self.params.get('VMAX')

    def get_reason_code(self):
        """ Get the merchant defined decision reason code. """
        return self.params.get("REASON_CODE")

    def get_score(self):
        """ Get the Kount score. """
        return self.params.get('SCOR')

    def getOmniscore(self):
        """ Get the Kount Omni Score"""
        return self.params.get('OMNISCORE')

    def get_geox(self):
        """ Get the geox. """
        return self.params.get('GEOX')

    def get_brand(self):
        """ Get the credit card brand."""
        return self.params.get('BRND')

    def get_network(self):
        """ Get the network type."""
        return self.params.get('NETW')

    def get_know_your_customer(self):
        """ Get the know your customer flag."""
        return self.params.get('KYCF')

    def get_region(self):
        """ Get the region the remote device is located in"""
        return self.params.get('REGN')

    def get_kaptcha(self):
        """ Get the Kaptcha flag: enabled upon request and for when RIS has
          no record. """
        return self.params.get("KAPT")

    def get_site(self):
        """ Get the site ID. """
        return self.params.get("SITE")

    def get_proxy(self):
        """ Get a string representing whether the remote device is using a
        proxy.
        :return: "Y" or "N"
        """
        return self.params.get("PROXY")

    def get_mails(self):
        """ Get the number of transactions associated with the email.
         :return Number of emails
         """
        return self.params.get("EMAILS")

    def get_http_country(self):
        """ Get the two character country code setting in the remote device's
         browser.

         :return Country
         """
        return self.params.get("HTTP_COUNTRY")

    def get_time_zone(self):
        """ Get a string representing the time zone of the customer
        as a 3 digit * number.
         :return Time zone
         """
        return self.params.get("TIMEZONE")

    def get_cards(self):
        """ Get the number of transactions associated with the credit card.

         :return Number of cards
         """
        return self.params.get("CARDS")

    def get_pc_remote(self):
        """ Get a string representing whether the end device is a remotely
         controlled computer.
         :returns "Y" or "N"
         """
        return self.params.get("PC_REMOTE")

    def get_devices(self):
        """ Get the number of transactions associated with the particular
         device.
         :return Number of devices
         """
        return self.params.get("DEVICES")

    def get_device_layers(self):
        """ Get a string representing the five layers (Operating System, SSL,
          HTTP, Flash, JavaScript) of the remote device.
         :return Device layers
         """
        return self.params.get("DEVICE_LAYERS")

    def get_mobile_forwarder(self):
        """ Get the mobile device's wireless application protocol.

         :return protocol
         """
        return self.params.get("MOBILE_FORWARDER")

    def get_voice_device(self):
        """ Get a string representing whether or not the remote device is
         voice controlled.
         :return "Y" or "N"
         """
        return self.params.get("VOICE_DEVICE")

    def get_local_time(self):
        """ Get local time of the remote device in the YYYY-MM-DD format.
         :return Local time
         """
        return self.params.get("LOCALTIME")

    def get_mobile_type(self):
        """ Get the mobile device type.
         :return Mobile type
         """
        return self.params.get("MOBILE_TYPE")

    def get_finger_print(self):
        """ * Get the device finger print.
         :return Finger print
        """
        return self.params.get("FINGERPRINT")

    def get_flash(self):
        """ * Get a string representing whether or not the remote device
        allows flash.
        :return "Y" or "N"
        """
        return self.params.get("FLASH")

    def get_language(self):
        """ Get the language setting on the remote device.
         :return Language
         """
        return self.params.get("LANGUAGE")

    def get_country(self):
        """ Get the remote device's country of origin as a two character code.
         :return Country
         """
        return self.params.get("COUNTRY")

    def get_java_script(self):
        """ Get a string representing whether the remote device allows
        JavaScript.
        :return "Y" or "N"
        """
        return self.params.get("JAVASCRIPT")

    def get_cookies(self):
        """ Get a string representing whether the remote device allows cookies.
        :return "Y" or "N"
        """
        return self.params.get("COOKIES")

    def get_mobile_device(self):
        """Get a string representing whether the remote device is a
        mobile device.
         :return "Y" or "N"
         """
        return self.params.get("MOBILE_DEVICE")

    def get_pierced_ip_address(self):
        """ Get pierced IP address.
         :return Pierced IP address
         """
        return self.params.get("PIP_IPAD")

    def get_pierced_ip_address_latitude(self):
        """ Get latitude of pierced IP address.
         :return Latitude of pierced IP address
         """
        return self.params.get("PIP_LAT")

    def get_pierced_ip_address_longitude(self):
        """ Get longitude of pierced IP address.
         :return Longitude of pierced IP address
         """
        return self.params.get("PIP_LON")

    def get_pierced_ip_address_country(self):
        """ Get country of pierced IP address.
         :return Country of pierced IP address
         """
        return self.params.get("PIP_COUNTRY")

    def get_pierced_ip_address_region(self):
        """ Get region of pierced IP address.
         :return Region of pierced IP address
         """
        return self.params.get("PIP_REGION")

    def get_pierced_ip_address_city(self):
        """ * Get city of pierced IP address.
         :return City of pierced IP address
         """
        return self.params.get("PIP_CITY")

    def get_pierced_ip_address_organization(self):
        """ Get organization of pierced IP address.
         :return Organization of pierced IP address
         """
        return self.params.get("PIP_ORG")

    def get_ip_address(self):
        """ Get proxy IP address.
         :return Proxy IP address
         """
        return self.params.get("IP_IPAD")

    def get_ip_address_latitude(self):
        """ Get latitude of proxy IP address.
         :return Latitude of proxy IP address
         """
        return self.params.get("IP_LAT")

    def get_ip_address_longitude(self):
        """ Get longitude of proxy IP address.
         :return Longitude of proxy IP address
         """
        return self.params.get("IP_LON")

    def get_ip_address_country(self):
        """ Get country of proxy IP address.
         :return Country of proxy IP address
         """
        return self.params.get("IP_COUNTRY")

    def get_ip_address_region(self):
        """ Get region of proxy IP address.
         :return Region of proxy IP address
         """
        return self.params.get("IP_REGION")

    def get_ip_address_city(self):
        """ Get city of proxy IP address.
         :return City of proxy IP address
         """
        return self.params.get("IP_CITY")

    def get_ip_address_organization(self):
        """Get organization of proxy IP address.
         :return Organization of proxy IP address
         """
        return self.params.get("IP_ORG")

    def get_date_device_first_seen(self):
        """ Get date device first seen.
         :return Date device first seen
         """
        return self.params.get("DDFS")

    def get_user_agent_string(self):
        """ Get user agent string.
         :return User agent string
         """
        return self.params.get("UAS")

    def get_device_screen_resolution(self):
        """ Get device screen resolution.
        :return Device screen resolution (HxW - Height by Width)
        """
        return self.params.get("DSR")

    def get_os(self):
        """ Get operating system (derived from user agent string).
        :return OS (operating system)
        """
        return self.params.get("OS")

    def get_browser(self):
        """ Get browser (derived from user agent string).
         :return Browser
         """
        return self.params.get("BROWSER")

    def get_kc_customer_id(self):
        """ Get the Kount Central Customer ID.
         :return String KC Id
         """
        return self.params.get("KC_CUSTOMER_ID")

    def get_warnings(self):
        """get_warnings"""
        warnings = []
        for i in range(int(self.params.get("WARNING_COUNT", 0))):
            warnings.append(self.params["WARNING_%s" % i])
        LOG.debug("RIS get_warnings = %s", warnings)
        return warnings

    def get_error_code(self):
        """ Get a possible error code."""
        return self.params.get('ERRO')

    def get_errors(self):
        """Get an ArrayList of errors associated with the response."""
        errors = []
        error_count = int(self.params.get("ERROR_COUNT", 0))
        for i in range(error_count):
            errors.append(self.params["ERROR_%s" % i])
        LOG.debug("RIS get_errors= %s", errors)
        return errors

    def get_lexis_nexis_cbd_attributes(self):
        """Get LexisNexis Chargeback Defender attribute data
        associated with the RIS
        transaction. Please contact your Kount representative to enable support
        for this feature in your merchant account.
        return Map of attributes where the keys are the attribute names and the
        values are the attribute values."""
        cbd = self.get_prefixed_response_data_map("CBD_")
        LOG.debug("get_lexis_nexis_cbd_attributes = %s", cbd)
        return cbd

    def get_lexis_nexis_instant_id_attributes(self):
        """Get LexisNexis Instant ID attributes associated with the
        RIS transaction.
        Please contact your Kount representative to enable support for this
        feature in your merchant account.
        return Map of attributes where the keys are the attribute names and the
        values are the attribute values."""
        instand_id = self.get_prefixed_response_data_map("INSTANTID_")
        LOG.debug("get_lexis_nexis_instand_id_attributes = %s", instand_id)
        return instand_id

    def get_mastercard_fraud_score(self):
        """ Get MasterCard Fraud Score associated with the RIS transaction.
        Please contact your Kount representative to enable support for this
        feature in your merchant account. """
        score = self.params.get('MASTERCARD')
        LOG.debug("get_mastercard_fraud_score = %s", score)
        return score

    def get_prefixed_response_data_map(self, prefix):
        """Get a map of the response data where the keys are the RIS response
        keys that begin with a specified prefix.
        Arg: prefix - Key prefix.
        return dict of key-value pairs for a specified RIS key prefix."""
        data = {key: self.params[key] for key in self.params
                if key.startswith(prefix)}
        LOG.debug("get_prefixed_response_data_map = %s", data)
        return data

    def get_counters_triggered(self):
        """Get a map of the rules counters triggered in the response.
        return dict Key: counter name, Value: counter value.
        """
        counters = {}
        num_counters = int(self.params.get("COUNTERS_TRIGGERED", 0))
        for i in range(num_counters):
            counters[self.params["COUNTER_NAME_%s" % i]] = \
                self.params["COUNTER_VALUE_%s" % i]
        LOG.debug("get_counters_triggered = %s", counters)
        return counters

    def get_version(self):
        """ Get the version number.
        :return RIS version
        """
        return self.params.get("VERS")

    def get_merchant(self):
        """ Get the merchant id.
        :return Merchant ID
        """
        return self.params.get("MERC")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""parse Kount's validation xml to python dict, used in RisValidator"""
# This file is part of the Kount python sdk project
# https://github.com/Kount/kount-ris-python-sdk/)
# Copyright (C) 2017 Kount Inc. All Rights Reserved.

from __future__ import absolute_import, unicode_literals, division, \
    print_function

import xml.etree.ElementTree as ET

from kount.version import VERSION
from kount.config import SDKConfig

__author__ = SDKConfig.SDK_AUTHOR
__version__ = VERSION
__maintainer__ = SDKConfig.SDK_MAINTAINER
__email__ = SDKConfig.MAINTAINER_EMAIL
__status__ = SDKConfig.STATUS


def xml_to_dict(xml_filename_path):
    "parse xml to_python dict"
    with open(xml_filename_path, 'r'):
        tree = ET.parse(xml_filename_path)
        root = tree.getroot()
    valid_data_dict = {}
    required_field_names = []
    notrequired_field_names = []
    for child in root:
        param_name = child.attrib['name']
        current = valid_data_dict[param_name] = {}
        required = child.find('required')
        if required is not None:
            required_field_names.append(param_name)
            current['required'] = True
            mode_tags = required.findall('mode')
            if mode_tags:
                current["mode"] = [m.text for m in mode_tags]
        else:
            notrequired_field_names.append(param_name)
        reg_ex = child.find('reg_ex')
        if reg_ex is not None:
            current["reg_ex"] = reg_ex.text
        max_length = child.find('max_length')
        if max_length is not None:
            current["max_length"] = max_length.text
    assert len(valid_data_dict.keys()) == len(root)
    return valid_data_dict, required_field_names, notrequired_field_names



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of the Kount python sdk project
# https://github.com/Kount/kount-ris-python-sdk/)
# Copyright (C) 2017 Kount Inc. All Rights Reserved.

import os

from . import resources
from .version import VERSION

__author__ = "Kount SDK"
__version__ = VERSION
__maintainer__ = "Kount SDK"
__email__ = "sdkadmin@kount.com"
__status__ = "Development"


class SDKConfig:
    # field validation rules
    # _XML_FILENAME = os.path.join(resources.__path__[0], 'validate.xml')

    SDK_VERSION = "0700"
    
    SDK_AUTHOR = __author__

    SDK_MAINTAINER = __maintainer__

    MAINTAINER_EMAIL = __email__

    STATUS = __status__
    # default behaviour for request failures
    _RAISE_ERRORS = True

    # requests timeout
    _DEFAULT_TIMEOUT = None

    # should be set from the sdk user
    _CONFIGURATION_KEY = None

    # @classmethod
    # def get_rules_xml_file(cls):
    #     return cls._XML_FILENAME

    @classmethod
    def get_default_timeout(cls):
        return cls._DEFAULT_TIMEOUT

    @classmethod
    def get_configuration_key(cls):
        return cls._CONFIGURATION_KEY

    @classmethod
    def get_should_raise_validation_errors(cls):
        return cls._RAISE_ERRORS

    @classmethod
    def setup(cls,
              config_key,
              default_timeout=5,
              raise_errors=True,
              xml_rules_file_name=None):
        """
        Call this method before start using the SDK
        :param config_key: mandatory parameter, configuration key provided
         by Kount
        :param default_timeout: request timeout, default value is 5 seconds
        :param raise_errors: indicate if the request should throw an exception
         in case of error, default value is True
        :param xml_rules_file_name: xml rules for validation of the request,
         should not be overwritten, unless you know what you are doing
        """

        cls._CONFIGURATION_KEY = config_key
        cls._DEFAULT_TIMEOUT = default_timeout
        cls._RAISE_ERRORS = raise_errors
        # if xml_rules_file_name:
        #     cls._XML_FILENAME = xml_rules_file_name

        from .util import khash
        k = khash.Khash(config_key)
        k.verify()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of the Kount python sdk project
# https://github.com/Kount/kount-ris-python-sdk/)
# Copyright (C) 2017 Kount Inc. All Rights Reserved.
"class Khash"

from __future__ import (absolute_import, unicode_literals,
                        division, print_function)

import hashlib
import logging
import re
import string

from kount.resources.correct_key_cryp import correct_key_cryp
from kount.version import VERSION
from .a85 import a85decode
from kount.config import SDKConfig

__author__ = SDKConfig.SDK_AUTHOR
__version__ = VERSION
__maintainer__ = SDKConfig.SDK_MAINTAINER
__email__ = SDKConfig.MAINTAINER_EMAIL
__status__ = SDKConfig.STATUS

LOG = logging.getLogger('kount.khash')


def validator(*args):
    """check is list with arguments is valid -
    positive integer or length of the string
    Args: list with integers or strings
    returns: list with args as strings
    """
    for current in args:
        try:
            str(current)
            is_string = True
        except ValueError:
            is_string = False
        curr_len = len(str(current))
        invalid = curr_len < 6 or curr_len > 100 or current is None
        try:
            current = int(current)
            is_digit = True
        except (TypeError, ValueError):
            is_digit = False
        if is_digit and int(current) <= 0:
            raise ValueError("incorrect arg: [%s]" % current)
        elif invalid or not is_string:
            raise ValueError("incorrect arg: [%s]" % current)
    return [str(i) for i in args]


class Khash(object):

    re_khashed = re.compile(r"^[0-9a-zA-Z]{6}[0-9A-Z]{14}$")
    __instance = None

    @staticmethod
    def get():
        """ Static access method. """
        SDKConfig.setup(SDKConfig._CONFIGURATION_KEY)
        # if Khash.__instance is None:
        #     raise ValueError('config.SDKConfig.setup() not called')
        return Khash.__instance

    """
    Class for creating Kount RIS KHASH encoding payment tokens.
    """
    def __init__(self, key):
        """ Virtually private constructor. """
        # if Khash.__instance is not None:
        #     pass
        #     raise Exception("This class is a singleton!")
        hash_salt_key = a85decode(key)
        self.config_key = hash_salt_key.decode("utf-8")
        self.verify()
        self.salt_key = hash_salt_key
        Khash.__instance = self

    def verify(self):
        key = self.config_key.encode('utf-8')
        sha_key = hashlib.sha256(key).hexdigest()
        # if sha_key != correct_key_cryp:
        #     mesg = "Configured config_key is incorrect."
        #     LOG.error(mesg)
        #     raise ValueError(mesg)
        LOG.info("Configured config_key is correct.")
        return True

    def hash_payment_token(self, token):
        """Create a Kount hash of a provided payment token. Payment tokens
        that can be hashed via this method include: credit card numbers,
        Paypal payment IDs, Check numbers, Google Checkout IDs, Bill Me
        Later IDs, and Green Dot MoneyPak IDs.

        Args: token - String to be hashed
        returns: String hashed
        if len(token) < 6 - need to clarify the expected behaviour
        """
        token_valid = validator(token)[0]
        return "%s%s" % (token_valid[:6], self.hash(plain_text=token_valid))

    def hash_gift_card(self, merchant_id, card_number):
        """ Hash a gift card payment token using the Kount hashing algorithm.
        Args: merchant_id - Merchant ID number
        card_number - Card number to be hashed
        returns: String hashed
        """
        merchant_id, card_number = validator(merchant_id, card_number)
        return "%s%s" % (merchant_id, self.hash(plain_text=card_number))

    def hash(self, plain_text):
        """
        Compute a Kount hash of a given plain text string.
        Preserves the first six characters of the input
        so that hashed tokens can be categorized
        by Bank Identification Number (BIN).
        Args: plain_text - String to be hashed
        returns: String hashed
        """
        if isinstance(plain_text, int):
            plain_text = str(plain_text)
        if validator(plain_text):
            legal_chars = string.digits + string.ascii_uppercase
            loop_max = 28
            hex_chunk = 7
            length = len(legal_chars)
            hashed = []
            plain_text_bytes = plain_text.encode('utf-8')  # Python 3.x
            sha1 = hashlib.sha1(plain_text_bytes + ".".encode('utf-8') +
                                self.salt_key).hexdigest()
            for i in range(0, loop_max, 2):
                hashed.append(legal_chars[int(sha1[i: i + hex_chunk], 16)
                                          % length])
            return ''.join(hashed)
        else:
            raise ValueError("incorrect arg: [%s]" % plain_text)

    @classmethod
    def khashed(cls, val):
        """ Arg: val - String, Token that may or may not be khashed
         return: Boolean, True if token is already khashed
        """
        return cls.re_khashed.match(val)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of the Kount python sdk project
# https://github.com/Kount/kount-ris-python-sdk
# Copyright (C) 2017 Kount Inc. All Rights Reserved.
"""class Client"""

from __future__ import (absolute_import, unicode_literals,
                        division, print_function)

import logging
import requests

from . import config
from . import request
from . import response
from .ris_validator import RisValidator, RisValidationException
from .version import VERSION
from .config import SDKConfig

__author__ = SDKConfig.SDK_AUTHOR
__version__ = VERSION
__maintainer__ = SDKConfig.SDK_MAINTAINER
__email__ = SDKConfig.MAINTAINER_EMAIL
__status__ = SDKConfig.STATUS


LOG = logging.getLogger('kount.client')


class Client:

    def __init__(self, api_url, api_key,
                 timeout=None,
                 raise_errors=None):
        """
        Client class used to execute Kount request
        :param api_url: endpoint to which the request should be sent
        :param api_key: merchant api key, provided by Kount
        :param timeout: request timeout, if not set the default value
         from SDKConfig will be used
        :param raise_errors: indicates if request validation error should
         be thrown, if not set the default value from SDKConfig will be used
        """
        self.api_url = api_url
        self.api_key = api_key
        conf = config.SDKConfig
        self.timeout = timeout or conf.get_default_timeout()
        # if raise_errors is None:
        #     raise_errors = conf.get_should_raise_validation_errors()
        # self.raise_errors = raise_errors
        # self.validator = RisValidator(conf.get_rules_xml_file(),
        #                               raise_errors)
        LOG.debug("url - %s, len_key - %s", self.api_url, len(self.api_key))

    def process(self, req):
        # if not isinstance(req, request.Request):
        #     raise ValueError("invalid request %s" % type(request))
        res = self._execute(req.params)
        if res:
            return response.Response(res)
        return None

    def _execute(self, params):
        """validate data and request post
        https://pypi.python.org/pypi/requests
        Use simplejson if available.
        if raise_errors==False, the validation errors will not be raised,
        only logged; by default raise_errors=True"""
        if LOG.isEnabledFor(logging.DEBUG):
            for key, param in params.items():
                LOG.debug('%s=%s', key, param)
        # invalid, missing_in_xml, empty = self.validator.ris_validator(
        #     params=params)
        # if invalid:
        #     message = "validation errors = %s, missing_in_xml = %s,"\
        #               "empty = %s" % (invalid, missing_in_xml, empty)
        #     LOG.error(message)
        #     if self.raise_errors:
        #         raise RisValidationException(
        #             message, errors=invalid, cause="empty = %s" % empty)
        headers_api = {'X-Kount-Api-Key': self.api_key}
        params['FRMT'] = 'JSON'
        LOG.debug("url=%s, headers=%s, params=%s", self.api_url,
                  headers_api, params)

        req = requests.post(self.api_url,
                            headers=headers_api,
                            data=params,
                            timeout=self.timeout)
        
        resp = req.json()
        text_to_json = parse_k_v(req.text)
        return text_to_json
        # try:
        #     resp = req.json()
        # except ValueError as jde:
        #     LOG.error("ValueError - %s", jde)
        #     try:
        #         text_to_json = parse_k_v(req.text)
        #         LOG.debug("process text: %s", text_to_json)
        #         return text_to_json
        #     except ValueError:
        #         error = "Neither JSON nor String %s" % req.text
        #         LOG.debug(error)
        #         raise ValueError(error)
        else:
            LOG.debug("MERC = %s, SESS = %s, SDK ELAPSED = %s ms.",
                      params.get('MERC', None),
                      params.get("SESS", None),
                      req.elapsed.total_seconds())
            return resp


def parse_k_v(text):
    """parse text to dict"""
    return dict(c.split('=', 1) for c in text.split('\n'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of the Kount python sdk project
# https://github.com/Kount/kount-ris-python-sdk/)
# Copyright (C) 2017 Kount Inc. All Rights Reserved.
"correct configurationKey - sha-256"

from kount.version import VERSION
from kount.config import SDKConfig

__author__ = SDKConfig.SDK_AUTHOR
__version__ = VERSION
__maintainer__ = SDKConfig.SDK_MAINTAINER
__email__ = SDKConfig.MAINTAINER_EMAIL
__status__ = SDKConfig.STATUS


correct_key_cryp = "ce1638f9cf6ce31b4ead12ca49712f1e538a055a6152ec83c7285e56ce917bd6"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of the Kount python sdk project
# https://github.com/Kount/kount-ris-python-sdk/)
# Copyright (C) 2017 Kount Inc. All Rights Reserved
"CartItem class"

from __future__ import absolute_import, unicode_literals, division, \
    print_function

from kount.version import VERSION
from kount.config import SDKConfig

__author__ = SDKConfig.SDK_AUTHOR
__version__ = VERSION
__maintainer__ = SDKConfig.SDK_MAINTAINER
__email__ = SDKConfig.MAINTAINER_EMAIL
__status__ = SDKConfig.STATUS


class CartItem(object):
    """A class that represents a shopping cart item.
    kwargs -
        product_type - the product type
        item_name - name of the item
        description - description
        quantity - quantity
        price - the price of the item
    """
    def __init__(self, product_type="", item_name="",
                 description="", quantity="", price=""):
        "Constructor for a cart item."
        self.product_type = product_type
        self.item_name = item_name
        self.description = description
        self.quantity = quantity
        self.price = price

    def __str__(self):
        "String representation of this shopping cart item"
        cart = "Product Type: %s\nItem Name: %s\nDescription:"\
               "%s\nQuantity: %s\nPrice: %s\n" % (
                   self.product_type, self.item_name, self.description,
                   self.quantity, self.price)
        return cart



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of the Kount python sdk project
# https://github.com/Kount/kount-ris-python-sdk/)
# Copyright (C) 2017 Kount Inc. All Rights Reserved.

from __future__ import (absolute_import, unicode_literals,
                        division, print_function)

from .version import VERSION
from kount.config import SDKConfig

__author__ = SDKConfig.SDK_AUTHOR
__version__ = VERSION
__maintainer__ = SDKConfig.SDK_MAINTAINER
__email__ = SDKConfig.MAINTAINER_EMAIL
__status__ = SDKConfig.STATUS


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of the Kount python sdk project
# https://github.com/Kount/kount-ris-python-sdk/)
# Copyright (C) 2017 Kount Inc. All Rights Reserved
"""RIS initial inquiry class"""
import time

import ipaddress
import logging
from datetime import datetime

from .config import SDKConfig
from .request import (Request, CurrencyType, InquiryMode,
                      Gender, AddressType, ShippingType)
from .util.address import Address
from .util.cartitem import CartItem
from .version import VERSION

__author__ = SDKConfig.SDK_AUTHOR
__version__ = VERSION
__maintainer__ = SDKConfig.SDK_MAINTAINER
__email__ = SDKConfig.MAINTAINER_EMAIL
__status__ = SDKConfig.STATUS


# common logger for inquiry and request
LOG = logging.getLogger('kount.request')


class Inquiry(Request):
    """RIS initial inquiry class.
        Contains specific methods for setting various inquiry properties
        Class constructor. Sets the RIS mode to "Inquiry" ("Q"),
        sets currency to "USD", and sets the Python SDK identifier.
        The mode and currency can be adjusted by called
        InquiryMode and CurrencyType methods respectively.
        """
    def __init__(self):
        super(Inquiry, self).__init__()
        self.version()
        self.params["SDK"] = "CUST"
        self.params["ANID"] = ""
        self.params["FRMT"] = "JSON"
        self.params["VERS"] = SDKConfig.SDK_VERSION
        self.inquiry_mode = InquiryMode.DEFAULT
        self.currency_type = CurrencyType.USD
        LOG.debug('Inquiry: %s', self.params)

    def version(self):
        """SDK_Type-RIS_VERSION-SDK_BUILD_DATETIMESTAMP"""
        datestr = datetime.now().strftime('%Y%m%d%H%M')
        vers = "Sdk-Ris-Python-%s-%s" % (SDKConfig.SDK_VERSION, datestr)
        assert len(vers) == 32
        self.set_param("SDK_VERSION", vers)

    def set_cash(self, cash):
        """Set cash amount of any feasible goods.
            Arg: cash - int, cash amount of any feasible goods"""
        self.set_param("CASH", cash)

    def set_date_of_birth(self, dob=datetime.today()):
        """Set the date of birth in the format YYYY-MM-DD.
         Arg: dob - Date of birth
         """
        self.set_param("DOB", dob.strftime('%Y-%m-%d'))

    def set_gender(self, gender):
        """Set the gender. Either M(ale) of F(emale).
            Acceptable values: GENDER
            Arg: - gender"""
        # if Gender.is_valid(gender):
        #     self.set_param("GENDER", gender)
        # else:
        #     raise ValueError('INVALID GENDER = %s' % gender)
        self.set_param("GENDER", gender)

    def set_user_defined_field(self, label, value):
        """Set a user defined field.
            Arg: label - The name of the user defined field
            Arg: value - The value of the user defined field
        """
        self.set_param("UDF[%s]" % label, value)

    def set_request_mode(self, mode):
        """Set the request mode.
            Acceptable values are: InquiryMode
            Arg: mode - Mode of the request
        """
        """if InquiryMode.is_valid(mode):
            self.set_param("MODE", mode)
        else:
            raise ValueError('INVALID INQUIRY MODE = %s' % mode)
        """
        self.set_param("MODE", mode)

    def set_currency(self, currency):
        """Set the three character ISO-4217 currency code.
            Arg: currency - Type of currency, CurrencyType
        """
        self.set_param("CURR", currency)

    def set_total(self, total=0):
        """Set the total amount in lowest possible denomination of currency.
            Arg: total - Transaction amount in lowest possible
            denomination of given currency
        """
        self.set_param("TOTL", total)

    def set_email_client(self, email_addr):
        """Set the email address of the client.
            Arg: email - Email address of the client
        """
        self.set_param("EMAL", email_addr)

    def set_customer_name(self, c_name):
        """the name of the client or company.
            Arg: c_name - Name of the client or company
         """
        self.set_param("NAME", c_name)

    def _address(self, adr_type, address):
        """Set the address.
            Arg: address - The billing or shipping address; type Address
            adr_type - billing or shipping, values in ['B', 'S']
        """
        if not isinstance(address, Address):
            raise ValueError
        self.params[adr_type + "2A1"] = address.address1
        self.params["%s2A2" % adr_type] = address.address2
        self.params["%s2CI" % adr_type] = address.city
        self.params["%s2ST" % adr_type] = address.state
        self.params["%s2PC" % adr_type] = address.postal_code
        self.params["%s2CC" % adr_type] = address.country
        self.params["%sPREMISE" % adr_type] = address.premise
        self.params["%sSTREET" % adr_type] = address.street

    def set_billing_address(self, address):
        """Set the billing address.
            Arg: address - The billing address, type Address
            Address
        """
        LOG.debug("B2A1 = %s, B2A2 = %s, B2CI = %s, B2ST = %s, "
                  "B2PC = %s, B2CC = %s, BPREMISE = %s, BSTREET = %s",
                  address.address1, address.address2, address.city,
                  address.state, address.postal_code, address.country,
                  address.premise, address.street)
        self._address(AddressType.BILLING, address)

    def set_shipping_address(self, address):
        """Set the shipping address.
            Arg: address - The shipping address, type Address
        """
        LOG.debug("S2A1 = %s, S2A2 = %s, S2CI = %s, S2ST = %s, "
                  "S2PC = %s, S2CC = %s, SPREMISE = %s, SSTREET = %s",
                  address.address1, address.address2, address.city,
                  address.state, address.postal_code, address.country,
                  address.premise, address.street)
        self._address(AddressType.SHIPPING, address)

    def set_billing_phone_number(self, billing_phone=""):
        """Set the billing phone number.
            Arg: billing_phone - Billing phone number
         """
        self.set_param("B2PN", billing_phone)

    def set_shipping_phone_number(self, shipping_phone):
        """Set the shipping phone number.
            Arg: shipping_phone - shipping phone number
         """
        self.set_param("S2PN", shipping_phone)

    def set_shipping_name(self, ship_name=""):
        """Set the shipping name.
            Arg: ship_name - Shipping name
        """
        self.set_param("S2NM", ship_name)

    def set_email_shipping(self, shipping_email):
        """Set the shipping email address of the client.
            Arg: shipping_email - shipping email
        """
        self.set_param("S2EM", shipping_email)

    def set_unique_customer_id(self, unique_customer):
        """Set the unique ID or cookie set by merchant.
           Arg: unique_customer - Customer-unique ID or cookie set by merchant.
        """
        self.set_param("UNIQ", unique_customer)

    def set_ip_address(self, ip_adr):
        """Set the IP address. ipaddress
        Arg: ip_adr - IP Address of the client
        """
        ipaddress.ip_address(ip_adr)
        self.set_param("IPAD", ip_adr)

    def set_user_agent(self, useragent):
        """Set the user agent string of the client.
        Arg: useragent - user agent string of the client
        """
        self.set_param("UAGT", useragent)

    def set_timestamp(self, time_stamp=None):
        """Set the timestamp (in seconds) since the UNIX epoch
            for when the UNIQ value was set.
            Arg: time_stamp -  The timestamp
        """
        if time_stamp is None:
            time_stamp = time.time()
        self.set_param("EPOC", time_stamp)

    def set_shipment_type(self, shipment):
        """Set shipment type
            Arg: shipment -  type ShippingType
        """
        if ShippingType.is_valid(shipment):
            self.set_param("SHTP", shipment)
        else:
            raise ValueError("INVALID SHIPMENT TYPE = %s" % shipment)

    def set_anid(self, anid_order):
        """Set the anid
            Automatic Number Identification (ANI) submitted with order.
            If the ANI cannot be determined,
            merchant must pass 0123456789 as the ANID.
            This field is only valid for MODE=P RIS submissions.
            Arg: anid_order - Anid of the client
        """
        self.set_param("ANID", anid_order)

    def set_company_name(self, name):
        """Set the name of the company.
        Arg: name - Name of the company
        """
        self.set_param("NAME", name)

    def set_website(self, web_site):
        """Set the website.
            Arg: site - the website
        """
        self.set_param("SITE", web_site)

    def set_shopping_cart(self, cart):
        """Set the shopping cart.
            Arg: cart - Cart items in the shopping cart, type Cart
        """
        for index, item in enumerate(cart):
            # if not isinstance(item, CartItem):
            #     raise ValueError('invalid CartItem: %s' % item)
            # LOG.debug("PROD_TYPE[%i] = %s, PROD_ITEM[%i] = %s, "
            #           "PROD_DESC[%i] = %s, PROD_QUANT[%i] = %s, "
            #           "PROD_PRICE[%i] = %s",
            #           index, item.product_type,
            #           index, item.item_name,
            #           index, item.description,
            #           index, item.quantity,
            #           index, item.price)
            self.params["PROD_TYPE[%i]" % index] = item.product_type
            self.params["PROD_ITEM[%i]" % index] = item.item_name
            self.params["PROD_DESC[%i]" % index] = item.description
            self.params["PROD_QUANT[%i]" % index] = item.quantity
            self.params["PROD_PRICE[%i]" % index] = item.price



#!/usr/bin/env python

# -*- coding: utf-8 -*-

"""

# This file is part of the Kount python sdk project

# https://github.com/Sanjeev139/python-hello-world/)

# Copyright (C) 2017 Kount Inc. All Rights Reserved.

"""

from __future__ import absolute_import



import os

from codecs import open

from setuptools import setup, find_packages



# GitLab env value will now dynamically set the version (gallilama)

if os.environ.get('CI_COMMIT_TAG'):

    version = os.environ['CI_COMMIT_TAG']

else:

    version = os.environ['CI_JOB_ID']
	



AUTHOR = 'Kount'

EMAIL = 'sanjeev1992jha@gmail.com'

PROJECT = 'uat_python_cicd'

PROJECT_MODULE = 'uat_python_cicd'

VERSION = version

PROJECT_URL = 'https://gitlab.com/gitlab-ci-cd-pipeine/pythoncicdproject'

DESC = 'Test ci/cd'

LONG_DESC = ''

LICENSE = 'Unknown'

KEYWORDS = ('python ci/cd', 'sanjeev python ci/cd')



# Version can be set dynamically by GitLab, but other things in the

# SDK appear to be using the value in version.py.

# Developers SHOULD STILL UPDATE the value in version.py for other needs.



this_path = os.path.abspath(os.path.dirname(__file__))



# Get the long description from the README file

with open(os.path.join(this_path, 'README.md'), encoding='utf-8') as f:

    LONG_DESC = f.read()



REQUIRES = (

    'requests>=2.11.1',

    'mom>=0.1.3;python_version<"3.0"',

    'py2-ipaddress>=3.4.1;python_version<"3.0"'

)



EXTRAS = {

    'test': (

        'pytest >= 2.8.4',

        'pytest-cov >= 2.4.0',

        'pytest-profiling >= 1.1.1',

        'pytest-html >= 1.14.2',

        'pytest-metadata >= 1.5.0',

    ),

    'dev': (

        'flake8 >= 2.5.0',

        'pytest >= 2.8.4',

        'pytest-cov >= 2.4.0',

        'pytest-profiling >= 1.1.1',

        'pylint >= 1.7.2',

        'pytest-html >= 1.14.2',

        'pytest-metadata >= 1.5.0',

    ),

}



# See https://pypi.python.org/pypi?%3Aaction=list_classifiers

CLASSIFIERS = [

    'Development Status :: 4 - Beta',

    'Intended Audience :: Developers',

    'Topic :: Software Development :: Build Tools',

    "Topic :: Internet :: WWW/HTTP",

    "Topic :: Software Development :: Libraries :: Python Modules",

    'License :: Other/Proprietary License',

    'Programming Language :: Python :: 2.7',

    'Programming Language :: Python :: 3.5',

    'Programming Language :: Python :: 3.6',

]



if __name__ == '__main__':

    setup(

        name=PROJECT,

        version=VERSION,

        description=DESC,

        long_description=LONG_DESC,

        long_description_content_type="text/markdown",

        url=PROJECT_URL,

        author=AUTHOR,

        author_email=EMAIL,

        license=LICENSE,

        classifiers=CLASSIFIERS,

        platforms=('any',),

        keywords=KEYWORDS,

        packages=find_packages('src'),

        package_dir={'': 'src'},

        install_requires=REQUIRES,

        setup_requires=['pytest-runner'],

        tests_require=['pytest'],

        extras_require=EXTRAS,

        # data_file=['kount/resources/validate.xml']

    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of the Kount python sdk project
# https://github.com/Kount/kount-ris-python-sdk/)
# Copyright (C) 2017 Kount Inc. All Rights Reserved.
from kount.version import VERSION


__author__ = "Kount SDK"
__version__ = VERSION
__maintainer__ = "Kount SDK"
__email__ = "sdkadmin@kount.com"
__status__ = "Development"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# This file is part of the Kount python sdk project
# https://github.com/Kount/kount-ris-python-sdk/)
# Copyright (C) 2017 Kount Inc. All Rights Reserved.
"VALIDATIONERROR, ValidationError"

from __future__ import absolute_import, unicode_literals, division, \
    print_function

from kount.version import VERSION
from kount.config import SDKConfig

__author__ = SDKConfig.SDK_AUTHOR
__version__ = VERSION
__maintainer__ = SDKConfig.SDK_MAINTAINER
__email__ = SDKConfig.MAINTAINER_EMAIL
__status__ = SDKConfig.STATUS


class ValidationError(Exception):
    _REGEX_ERR = 'REGEX'
    _LENGTH_ERR = 'LENGTH'
    _REQUIRED_ERR = 'REQUIRED'
    """Get the string representation of the error.
    Keyword arguments:
    field - the name of the bad field
    mode - the RIS mode the field is associated with
    value - field value
    pattern - the regular expression violated
    length - the maximum allowable length
    raise  ValidationError
    """
    def __init__(self, field="", mode="", value="", pattern="", length=0):
        self.error = None
        self.message = None
        if 0 < length < len(field):
            self.message = "Field [%s] has length [%s] which is longer than "\
                           "the maximum of [%s]" % (field, len(field), length)
            self.error = self._LENGTH_ERR
        elif pattern:
            self.message = "Field [%s] has value [%s] which does not "\
                           "match the pattern [%s]" % (field, value, pattern)
            self.error = self._REGEX_ERR
        elif mode:
            self.message = "Required field [%s] missing for mode [%s]" % (
                field, mode.upper())
            self.error = self._REQUIRED_ERR
        if not self.error or not self.message:
            raise RuntimeError("ValidationError not set")
        super(ValidationError, self).__init__(self.message)
